---- tern: disable-tx ----
-- disable migration transaction, to allow for batched updates

-- jsonb does not support unicode escaped null bytes, and we don't
-- control much of the output generated by osbuild.
ALTER TABLE jobs ADD COLUMN IF NOT EXISTS new_result json;

-- backfill outside of the transaction to avoid locking the table
DO $$
DECLARE
  remaining_rows INTEGER := 1;
BEGIN
  WHILE remaining_rows > 0 LOOP
    -- CTE on jobs
    WITH cte_jobs AS (
       SELECT ctid
         FROM jobs
         WHERE result IS NOT NULL AND new_result IS NULL
         FOR UPDATE
         LIMIT 500
    )
    -- self-join on ctid
    UPDATE jobs set new_result = result::json
    FROM cte_jobs
    WHERE jobs.ctid = cte_jobs.ctid;

    SELECT COUNT(ctid)
    FROM jobs
    WHERE result IS NOT NULL AND new_result IS NULL
    INTO remaining_rows;
    RAISE NOTICE 'REMAINING ROWS: %', remaining_rows;
  END LOOP;
END $$;

-- switching over the columns needs to happen in a transaction
BEGIN;

DROP VIEW ready_jobs;
ALTER TABLE jobs DROP COLUMN result;
ALTER TABLE jobs RENAME COLUMN new_result TO result;

CREATE VIEW ready_jobs AS
SELECT *
FROM jobs
WHERE started_at IS NULL
  AND canceled = FALSE
  AND id NOT IN (
    SELECT job_id
    FROM job_dependencies JOIN jobs ON dependency_id = id
    WHERE finished_at IS NULL
)
ORDER BY queued_at ASC;
COMMIT;
